# automatically generated by the FlatBuffers compiler, do not modify

# namespace: FB_Tables


class DType(object):
    Int8 = 0
    Uint8 = 1
    Int16 = 2
    Uint16 = 3
    Int32 = 4
    Uint32 = 5
    Int64 = 6
    Uint64 = 7
    Float32 = 8
    Float64 = 9
    c_string = 10
