# automatically generated by the FlatBuffers compiler, do not modify

# namespace:


class Value(object):
    NONE = 0
    Byte = 1
    UByte = 2
    Short = 3
    UShort = 4
    Int = 5
    UInt = 6
    Long = 7
    ULong = 8
    Float = 9
    Double = 10
    ArrayByte = 11
    ArrayUByte = 12
    ArrayShort = 13
    ArrayUShort = 14
    ArrayInt = 15
    ArrayUInt = 16
    ArrayLong = 17
    ArrayULong = 18
    ArrayFloat = 19
    ArrayDouble = 20
    String = 21
    ArrayString = 22
