# automatically generated by the FlatBuffers compiler, do not modify

# namespace:


class Value(object):
    NONE = 0
    Int8 = 1
    UInt8 = 2
    Int16 = 3
    UInt16 = 4
    Int32 = 5
    UInt32 = 6
    Int64 = 7
    UInt64 = 8
    Float32 = 9
    Float64 = 10
    ArrayInt8 = 11
    ArrayUInt8 = 12
    ArrayInt16 = 13
    ArrayUInt16 = 14
    ArrayInt32 = 15
    ArrayUInt32 = 16
    ArrayInt64 = 17
    ArrayUInt64 = 18
    ArrayFloat32 = 19
    ArrayFloat64 = 20
