# automatically generated by the FlatBuffers compiler, do not modify

# namespace:


class DType(object):
    int8 = 0
    uint8 = 1
    int16 = 2
    uint16 = 3
    int32 = 4
    uint32 = 5
    int64 = 6
    uint64 = 7
    float32 = 8
    float64 = 9
    c_string = 10
