# automatically generated by the FlatBuffers compiler, do not modify

# namespace:


class ValueUnion(object):
    NONE = 0
    Int8Array = 1
    UInt8Array = 2
    Int16Array = 3
    UInt16Array = 4
    Int32Array = 5
    UInt32Array = 6
    Int64Array = 7
    UInt64Array = 8
