# automatically generated by the FlatBuffers compiler, do not modify

# namespace:


class EventType(object):
    UNKNOWN = 0
    NEVER_CONNECTED = 1
    CONNECTED = 2
    DISCONNECTED = 3
    DESTROYED = 4
