# automatically generated by the FlatBuffers compiler, do not modify

# namespace:


class AlarmState(object):
    NONE = 0
    DEVICE = 1
    DRIVER = 2
    RECORD = 3
    DB = 4
    CONF = 5
    UNDEFINED = 6
    CLIENT = 7
